Package: googledrive
Title: An Interface to Google Drive
Version: 2.1.2
Authors@R: c(
    person("Lucy", "D'Agostino McGowan", , role = "aut"),
    person("Jennifer", "Bryan", , "jenny@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-6983-2759")),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Manage Google Drive files from R.
License: MIT + file LICENSE
URL: https://googledrive.tidyverse.org,
        https://github.com/tidyverse/googledrive
BugReports: https://github.com/tidyverse/googledrive/issues
Depends: R (>= 4.1)
Imports: cli (>= 3.0.0), gargle (>= 1.6.0), glue (>= 1.4.2), httr,
        jsonlite, lifecycle, magrittr, pillar (>= 1.9.0), purrr (>=
        1.0.1), rlang (>= 1.0.2), tibble (>= 2.0.0), utils, uuid, vctrs
        (>= 0.3.0), withr
Suggests: curl, dplyr (>= 1.0.0), knitr, rmarkdown, spelling, testthat
        (>= 3.1.5)
VignetteBuilder: knitr
Config/Needs/website: tidyverse, tidyverse/tidytemplate
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2025-09-10 06:10:03 UTC; jenny
Author: Lucy D'Agostino McGowan [aut],
  Jennifer Bryan [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6983-2759>),
  Posit Software, PBC [cph, fnd]
Maintainer: Jennifer Bryan <jenny@posit.co>
Repository: CRAN
Date/Publication: 2025-09-10 08:50:06 UTC
